<?php

use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Drupal\workflows\State;

/**
 * @file
 * Theme for Kanban views.
 */
function template_preprocess_views_view_kanban(&$variables) {
  $view = $variables['view'];
  $rows = $variables['rows'];
  $style = $view->style_plugin;
  $options = $style->options;
  $variables["view_id"] = $view->storage->id();
  $variables["display_id"] = $variables["view"]->current_display;

  $colors = [
    'primary',
    'warning',
    'success',
    'danger',
    'info',
    'dark',
    'secondary',
  ];

  $variables['view']->element['#attached']['library'][] = 'views_kanban/kanban';
  $variables['view']->element['#attached']['library'][] = 'core/drupal.dialog.ajax';
  $columns = [];

  //Dialog width
  $dialog_width = !empty($options["dialog_width"]) ? $options["dialog_width"] : '80%';
  if (!is_numeric($dialog_width)) {
    $dialog_width = '"' . $dialog_width . '"';
  }
  $entity_type_id = 'node';
  if (!empty($options["status_field"]) && !empty($rows)) {
    $entity = current($rows)->_entity;
    $entity_type_id = $entity->getEntityTypeId();
    $createNew = 'admin/people/create';
    if($entity_type_id != 'user'){
      $entityType = $entity->getType();
      $createNew = $entity_type_id . '/add/'.$entityType;
    }
    $extractStatus= explode(':',$options["status_field"]);
    if(!empty($extractStatus[1])){
      $options["status_field"] = $extractStatus[0];
      $options["workflow_id"] = $extractStatus[1];
    }
    $status_field = $entity->get($options["status_field"]);
    $field_status_settings = $status_field->getFieldDefinition()->getSettings();
    $status_values = [];
    if (!empty($field_status_settings["allowed_values"])) {
      $status_values = $field_status_settings["allowed_values"];
      $prepopulate = 'edit' . '[' . $options["status_field"] . '][widget][0][value]';
    }
    if (!empty($field_status_settings["target_type"]) && $field_status_settings["target_type"] == 'taxonomy_term') {
      $vid = current($field_status_settings["handler_settings"]["target_bundles"]);
      $entity_storage = \Drupal::entityTypeManager()
        ->getStorage($field_status_settings["target_type"]);
      $query_result = $entity_storage->getQuery()
        ->condition('vid', $vid)
        ->sort('weight', 'ASC')
        ->execute();
      // Load the terms.

      $terms = $entity_storage->loadMultiple($query_result);
      foreach ($terms as $term) {
        $status_values[$term->id()] = $term->getName();
      }
      $prepopulate = 'edit' . '[' . $options["status_field"] . '][widget][0][target_id]';
    }
    if(!empty($options["workflow_id"])){
      $workflow = \Drupal::entityTypeManager()->getStorage('workflow')->load($options["workflow_id"]);
      $status_values = array_map([State::class, 'labelCallback'], $workflow->getTypePlugin()->getStates());
    }
    $i = -1;
    foreach ($status_values as $id => $status_value) {
      if (empty($colors[++$i])) {
        $i = 0;
      }
      $linkOptions = [
        'query' => [$prepopulate => $id],
        'attributes' => ['class' => ['btn', 'btn-' . $colors[$i]]],
        'absolute' => TRUE,
      ];
      $columns[$id] = [
        'header' => $status_value,
        'color' => $colors[$i],
        'add' => [
          '#title' => t('Add'),
          '#type' => 'link',
          '#attributes' => [
            'class' => ['use-ajax'],
            'data-dialog-type' => 'modal',
            'data-dialog-options' => '{"width": ' . $dialog_width . '}',
          ],
          '#url' => Url::fromUri('internal:/' . $createNew, $linkOptions),
        ],
        'rows' => [],
      ];
    }
  }
  $variables['default_row_class'] = !empty($options['default_row_class']);

  $linkViewOptions = [
    'attributes' => [
      'class' => ['use-ajax', 'btn', 'btn-primary'],
      'data-dialog-type' => 'modal',
      'data-dialog-options' => '{"width": ' . $dialog_width . '}',
    ],
    'absolute' => TRUE,
  ];
  foreach ($rows as $id => $row) {
    $entity = $row->_entity;
    $status = current($entity->get($options["status_field"])->getValue());
    $statusValue = 0;
    if (!empty($field_status_settings["target_type"]) && $field_status_settings["target_type"] == 'taxonomy_term') {
      $statusValue = $status['target_id'];
    }
    if (!empty($status['value'])) {
      $statusValue = $status['value'];
    }
    $linkViewOptions['attributes']['class'][2] = 'btn-' . $columns[$statusValue]['color'];
    $entity_id = $entity->id();
    $entity_type = $entity->getEntityType()->id();
    if($entity_type_id != 'user'){
      $authorUid = $entity->getOwnerID();
    }else{
      $authorUid = $entity->id();
    }
    $author = User::load($authorUid);
    $assignValues = [$authorUid];
    $variables['rows'][$id] = [
      'entity_id' => $entity_id,
      'entity_type' => $entity_type,
      'attributes' => new Attribute(),
      'content' => [
        '#row' => $row,
        '#view' => $view,
        '#options' => ['default_field_elements' => FALSE],
        '#theme' => [
          'views_view_fields__kanban',
          'views_view_fields',
        ],
      ],
      'view' => $entity->toLink(t('View'), 'canonical', $linkViewOptions),
      'edit' => $entity->toLink(t('Edit'), "edit-form", $linkViewOptions),
      'author' => $author->getDisplayName(),
      'date' => \Drupal::service('date.formatter')
        ->format($entity->getCreatedTime(), 'short'),
    ];
    //Title
    if (!empty($options["title_field"])) {
      $variables['rows'][$id]['title'] = $style->getField($id, $options["title_field"]);
    }
    //progress
    if (!empty($options["progress_field"])) {
      $variables['rows'][$id]['progress'] = $style->getFieldValue($id, $options["progress_field"]);
    }

    if ($row_class = $view->style_plugin->getRowClass($id)) {
      $variables['rows'][$id]['attributes']->addClass($row_class);
    }

    if (!empty($options["assign_field"])) {
      $assignValues = array_merge($assignValues, $style->getFieldValue($id, $options["assign_field"]));
      $assignValues = array_unique($assignValues);
    }
    // get user picture
    foreach ($assignValues as $uid) {
      $assignor = User::load($uid);
      //initial name
      $extractName = explode(' ', $assignor->getDisplayName());
      $acronym = $extractName[0][0];
      if (count($extractName) > 1) {
        $acronym .= end($extractName)[0];
      }
      $assign = [
        'name' => $assignor->getDisplayName(),
        'uid' => $uid,
        'acronym' => mb_strtoupper($acronym),
      ];
      if (!empty($assignor->user_picture) && !$assignor->user_picture->isEmpty()) {
        $avatarImg = \Drupal::service('file_url_generator')->generateAbsoluteString($assignor->user_picture->entity->getFileUri());
        $style = \Drupal::entityTypeManager()->getStorage('image_style')->load('thumbnail');
        $assign['avatar'] = $style->buildUrl($avatarImg);
      }
      $variables['rows'][$id]['assign'][] = $assign;
    }
    $columns[$statusValue]['rows'][$id] = $variables['rows'][$id];
  }
  $variables['columns'] = $columns;
}
